#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains constants definitions used by tpRigToolkit-tools-rigtoolbox
"""

from __future__ import print_function, division, absolute_import

TOOL_ID = 'tpRigToolkit-tools-rigtoolbox'
